//    ******************************************
//
//          Balance Room Display
//
//         author: jochen viehoff / lab3
//
//          * MultiCast communication
//          * Java 3D
//
//    ******************************************

import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.Frame;
import com.sun.j3d.utils.image.TextureLoader;
import java.awt.GraphicsConfiguration;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.ColorCube;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.Box;
import javax.media.j3d.*;
import javax.vecmath.*;

import java.awt.event.*;
import java.util.Enumeration;

public class BalanceRoomDisplay extends Applet {
    
    private java.net.URL image = null;
    private TextureUnitState textureUnitState;
    private Texture imageTex;
    
    public class MultiCastBehavior extends Behavior{
        
        private TransformGroup targetTG;
        
        // free parameter for fine tuning
        private float phiX      = 0.0f , phiY    = 0.0f;
        private float L         = 1.0f;
        private float offCenter = 0.85f;
        private float rotX      = 0.0f , rotY    = 0.0f;
        private float signX     = 1.0f , signY   = 1.0f;
        private float xOffset   = 0.0f , yOffset = 0.0F , zOffset = 0.0f;
        
        // data from multicast
        private float f1 = 0.0f,f2 = 0.0f,f1Prev,f2Prev;
        
        private WakeupCondition my_WakeupCondition = null;
        private MultiCastClient myClient;                    // Multicast Client
        
        // create SimpleBehavior
        MultiCastBehavior(TransformGroup targetTG){
            
            WakeupCriterion criterionArray[] = new WakeupCriterion[1];
            criterionArray[0] = new WakeupOnElapsedTime(1);
            
            my_WakeupCondition = new WakeupOr( criterionArray );
            this.targetTG = targetTG;
            MultiCastClient myClient = new MultiCastClient();
            
        }
        
        // initialize the Behavior
        //     set initial wakeup condition
        //     called when behavior beacomes live
        public void initialize(){
            this.wakeupOn( my_WakeupCondition );
        }
        
        // behave
        // called by Java 3D when appropriate stimulus occures
        public void processStimulus(Enumeration criteria){
            
            Transform3D rotation    = new Transform3D();
            Transform3D rotation2   = new Transform3D();
            Transform3D translation = new Transform3D();
            
            f1Prev = f1;
            f2Prev = f2;
            
            f1=myClient.getFloat1(1);
            f2=myClient.getFloat2(1);
            
            if ((f1!=f1Prev)||(f2!=f2Prev)){
                if (f1<100.0f) {
                    phiX = f1;
                    phiY = f2;
                }
                else if (f1==100.0f)
                    xOffset = xOffset+0.01f;
                else if (f1==101.0f)
                    xOffset = xOffset-0.01f;
                else if (f1==102.0f)
                    yOffset = yOffset+0.01f;
                else if (f1==103.0f)
                    yOffset = yOffset-0.01f;
                else if (f1==104.0f)
                    zOffset = zOffset+0.01f;
                else if (f1==105.0f)
                    zOffset = zOffset-0.01f;
                else if (f1==106.0f)
                    signX = signX+0.1f;
                else if (f1==107.0f)
                    signX = signX-0.1f;
                else if (f1==108.0f)
                    signY = signY+0.1f;
                else if (f1==109.0f)
                    signY = signY-0.1f;
                else if (f1==110.0f)
                    rotX = rotX+0.01f;
                else if (f1==111.0f)
                    rotX = rotX-0.01f;
                else if (f1==112.0f)
                    rotY = rotY+0.01f;
                else if (f1==113.0f)
                    rotY = rotY-0.01f;
                else if (f1==114.0f)
                    L = L+0.1f;
                else if (f1==115.0f)
                    L = L-0.1f;
            }
            translation.setTranslation(new Vector3f(0.0f,offCenter,0.0f));
            rotation.rotX(rotX+signX*phiX);
            rotation.mul(translation);
            rotation2.rotZ(rotY+signY*phiY);
            rotation2.mul(rotation);
            translation.setTranslation(new Vector3f(xOffset,yOffset-offCenter+phiX*L,zOffset));
            translation.mul(rotation2);
            targetTG.setTransform(translation);
            this.wakeupOn(my_WakeupCondition);
        }
    }
    public BranchGroup createSceneGraph() {
        // Create the root of the branch graph
        BranchGroup objRoot = new BranchGroup();
        
        TransformGroup objRotate = new TransformGroup();
        objRotate.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        
        Appearance ap = new Appearance();
        
        // load textures
        TextureAttributes texAttr = new TextureAttributes();
        texAttr.setTextureMode(TextureAttributes.DECAL);
        TextureLoader tex = new TextureLoader(image, new String("RGB"), this);
        imageTex = tex.getTexture();
        ap.setTexture(imageTex);
        
        Box BoxObj = new Box(1.2f, 0.85f, 0.0f, Box.GENERATE_NORMALS |
        Box.GENERATE_TEXTURE_COORDS, ap,2);
        
        objRoot.addChild(objRotate);
        objRotate.addChild(BoxObj);
        
        MultiCastBehavior myRotationBehavior = new MultiCastBehavior(objRotate);
        myRotationBehavior.setSchedulingBounds(new BoundingSphere());
        objRoot.addChild(myRotationBehavior);
        
        // Let Java 3D perform optimizations on this scene graph.
        objRoot.compile();
        
        return objRoot;
    } // end of CreateSceneGraph method of SimpleBehaviorApp
    
    // Create a simple scene and attach it to the virtual universe
    
    public BalanceRoomDisplay(java.net.URL imageURL) {
        image = imageURL;
        setLayout(new BorderLayout());
        GraphicsConfiguration config =
        SimpleUniverse.getPreferredConfiguration();
        
        Canvas3D canvas3D = new Canvas3D(config);
        add("Center", canvas3D);
        
        BranchGroup scene = createSceneGraph();
        
        // SimpleUniverse is a Convenience Utility class
        SimpleUniverse simpleU = new SimpleUniverse(canvas3D);
        
        // This will move the ViewPlatform back a bit so the
        // objects in the scene can be viewed.
        simpleU.getViewingPlatform().setNominalViewingTransform();
        
        simpleU.addBranchGraph(scene);
    } // end of SimpleBehaviorApp (constructor)
    //  The following allows this to be run as an application
    //  as well as an applet
    
    public static void main(String[] args) {
        java.net.URL imageURL = null;
        try {
            imageURL = new java.net.URL("file:C:/User/VideoBalance/gradeaus.jpg");
        }
        catch (java.net.MalformedURLException ex) {
            System.exit(1);
        }
        Frame frame = new MainFrame(new BalanceRoomDisplay(imageURL), 800, 600);
    }
}