// *******************************************************
// *                                                     *
// *    JAVA  Applikation:   BalanceRoomServer           *
// *                                                     *
// *                                                     *
// *    sample data from WeightPad                       *
// *    integrate equations of motion (symplectic)       *
// *    for 2D platform                                  *
// *    multicast network communication                  *
// *                                                     *
// *                                                     *
// *    author: jochen viehoff / lab3                    *
// *                                                     *
// *******************************************************

import java.applet.*;                          //  hier werden die Systemroutinen
import java.awt.*;                             //  geladen
import java.awt.Frame;                             //  geladen
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import java.net.*;
import java.io.*;


public class BalanceRoomServer implements Runnable{
    
    final double  a[] = new double [3];      // Konstanten fuer die Integration
    final double  b[] = new double [3];
    
    double weightA,weightB,weightC,weightD;         // weight cells
    
    double posAX = -1.0 , posAY =  1.0;             // positionen der waegezellen
    double posBX =  1.0 , posBY =  1.0;
    double posCX = -1.0 , posCY = -1.0;
    double posDX =  1.0 , posDY = -1.0;
    
    double phiX=0.0;                       // Winkel X
    double phiY=0.0;                       // Winkel Y
    double comX,comY;                      // com: center of mass - Schwerpunkt
    double mTotal;                         // Gesamtgewicht auf der Plattfom
    
    double veloX,veloY;                        // Geschwindigkeit
    double forceX,forceY;                      // Kraefte
    
    double I           = 0.5;               // Traegheitsmoment
    double dt          = 0.01;                  // Zeitschritt
    
    Thread animThread;
    Platform balanceFloor;
    int myID = 1;
    MultiCastServer balanceServer;
    
    int delay  = 5;
    
    //    ******************************************
    //      main method in class Integrate
    //    ******************************************
    
    public static void main(String[] args) {
        
        BalanceRoomServer myRoom = new BalanceRoomServer();
    }
    
    
    //    ******************************************
    //      constructor of Integrate2
    //    ******************************************
    
    public BalanceRoomServer() {
        
        // start multicast server
        MultiCastServer BalanceServer = new MultiCastServer(myID);
        
        // start labpro sampling
        Platform BalanceFloor = new Platform();
        
        a[0] =  7.0/24.0;                         // Konstanten fuer die Integration
        a[1] =  3.0/4.0;
        a[2] = -1.0/24.0;
        b[0] =  2.0/3.0;
        b[1] = -2.0/3.0;
        b[2] =  1.0;
        
        // start animation thread
        animThread = new Thread(this);
        animThread.start();
        
    }
    
    //    ******************************************
    //      Thread: run method
    //    ******************************************
    
    public void run() {
        
        while (animThread!=null){
            
            try {Thread.sleep(delay);}
            catch (InterruptedException e) {}
            
            weightA = balanceFloor.getA();
            weightB = balanceFloor.getB();
            weightC = balanceFloor.getC();
            weightD = balanceFloor.getD();
            
            mTotal = weightA+weightB+weightC+weightD;
            if (mTotal!=0.0){
                comX = (posAX*weightA+posBX*weightB+posCX*weightC+posDX*weightD)/mTotal;
                comY = (posAY*weightA+posBY*weightB+posCY*weightC+posDY*weightD)/mTotal;
            }
            iterate();                                     // Integration
            balanceServer.update((float)phiX,(float)phiY);
        }
    }
    
    // ****************************************************
    //   Berechnung der Kraefte
    // ****************************************************
    
    public void getForces() {
        //           Drehmoment von com        Rueckstellkraft      Reibung
        forceX = mTotal*I*comX*Math.cos(phiX)-100.0*Math.pow(phiX,3)-4.0*veloX;
        forceY = mTotal*I*comY*Math.cos(phiY)-100.0*Math.pow(phiY,3)-4.0*veloY;
    }
    
    // ****************************************************
    //   Integration: symplektisch 3. Ordnung
    // ****************************************************
    
    public void iterate() {
        
        for (int k=0;k<3;k++) {
            phiX += a[k]*dt*veloX;
            phiY += a[k]*dt*veloY;
        }
        
        getForces();		       // Berechnung der neuen Kraefte
        
        for (int k=0;k<3;k++) {
            //             veloX  += b[k]*dt*forceX;
            //             veloY  += b[k]*dt*forceY;
            veloX  += b[k]*dt*forceX;
            veloY  += b[k]*dt*forceY;
            
        }
    }
    
}
// ******************************************************
// ******************************************************

