// ******************************************************
//
//      Class:  MultiCastClient
//
//              Multicast client application
//              for two float streams with ID
//
//      protocol:  (byte) myID, (float) f1,(float) f2
//
//
//      author: jochen viehoff
//
// ******************************************************

import java.io.*;
import java.net.*;

public class MultiCastClient implements Runnable {
    
    
    // ******************************************************
    //  Declaration
    // ******************************************************
    
    public static int    PortNumber = 9377;              // port for multicast
    public static String IPAddress = "224.0.0.177";      // multicast register
    public static int    Participants = 10;              // max number of participants
    
    private static float f1,f2;
    private static float [] F1 = new float [Participants];
    private static float [] F2 = new float [Participants];
    
    private static byte  myID;
    private static int Mask = 0xFF;                      // sign mask for float reconstruction
    
    private static byte [] recBuffer = new byte[9];      // buffer for multicast data
    
    private int recDelay    = 1;                        // receiving delay
    
    private Thread MCThread;
    
    
    // ****************************************************
    //  constructor
    // ****************************************************
    
    public MultiCastClient(){
        if (MCThread == null) {
            MCThread = new Thread(this);	        // start MultiCast Thread
            MCThread.start();
        }
        
    }
    
    
    // ****************************************************
    //  MCThread: run Methode
    // ****************************************************
    
    public void run() {
        
        try {
            // open multicast socket
            
            MulticastSocket socket = new MulticastSocket(PortNumber);
            InetAddress group = InetAddress.getByName(IPAddress);
            socket.setInterface(InetAddress.getLocalHost());
            socket.joinGroup(group);
            System.out.println("time to live:"+socket.getTimeToLive());
            System.out.println("interface:"+socket.getInterface());
            
            byte [] b = new byte[9];
            DatagramPacket packet = new DatagramPacket(b,b.length);
            
            System.out.println("Start MultiCast Client.");
            
            while(MCThread != null) {                   // loop for receiving data
                socket.receive(packet);
                byte [] data = packet.getData();        // get the package
                myID = data[0];                         // get identity of package
                int intF1 = ((Mask & data[1])) | ((Mask & data[2])<<8) | ((Mask & data[3])<<16) |
                ((Mask & data[4])<<24);
                int intF2 = ((Mask & data[5])) | ((Mask & data[6])<<8) | ((Mask & data[7])<<16) |
                ((Mask & data[8])<<24);
                
                f1 = Float.intBitsToFloat(intF1);
                f2 = Float.intBitsToFloat(intF2);
                
                for (int i=0;i<Participants;i++) {       // archive data
                    if (myID==i) {
                        F1[i] = f1;
                        F2[i] = f2;
                    }
                }
                try {
                    Thread.sleep(recDelay);            // sleep for a while
                }
                catch(InterruptedException e) {}
                
            }
        }
        catch(IOException e) {
            e.printStackTrace();
        }
        
        MCThread = null;	                           // terminate thread
    }
    
    // ****************************************************
    //   getFloat1 of participant id
    // ****************************************************
    
    public static float getFloat1(int id) {
        
        return  F1[id];
    }
    // ****************************************************
    //   getFloat2 of participant id
    // ****************************************************
    
    public static float getFloat2(int id) {
        
        return F2[id];
    }
}