// ******************************************************
//
//      Class:  MultiCastServer (myID)
//
//              Multicast server application
//              for two float streams with ID
//
//      protocol:  (byte) myID, (float) f1,(float) f2
//
//
//      author: jochen viehoff
//
// ******************************************************


import java.io.*;
import java.net.*;

public class MultiCastServer extends Thread {
    
    
    // ******************************************************
    //  Declaration
    // ******************************************************
    
    public static int    PortNumber = 9377;
    public static String IPAddress = "224.0.0.177";
    
    float f1,f2;
    
    private static byte  myID;
    
    private static byte [] sendBuffer = new byte[9];
    
    private int sendDelay    = 1;
    private static boolean newData=false;
    
    private Thread MCThread;
    
    
    // ****************************************************
    //  constructor
    // ****************************************************
    
    public MultiCastServer(int id){
        
        myID = (byte) id;
        System.out.println("MCServer:"+myID);
        update(1.0f,1.0f);                          // init sendBuffer
        
        if (MCThread == null) {
            MCThread = new Thread(this);	        // start MultiCast Thread
            MCThread.start();
        }
        
    }
    
    
    // ****************************************************
    //  MCThread: run Methode
    // ****************************************************
    
    public void run() {
        
        System.out.println(myID);
        try {
            MulticastSocket socket = new MulticastSocket();
            InetAddress group = InetAddress.getByName(IPAddress);
            socket.setInterface(InetAddress.getLocalHost());
            socket.joinGroup(group);
            
            System.out.println("Start MultiCast Server, ID="+myID);
            
            while(MCThread != null) {                   // loop for sending data
                DatagramPacket out = new DatagramPacket(sendBuffer,sendBuffer.length,group,PortNumber);
                if (newData){
                    newData=false;
                    socket.send(out);
                }
                try {
                    Thread.sleep(sendDelay);            // sleep for a while
                }
                catch(InterruptedException e) {}
                
            }
        }
        catch(IOException e) {
            e.printStackTrace();
        }
        
        MCThread = null;
    }
    
    // ****************************************************
    //   update sendBuffer
    // ****************************************************
    
    public static void update(float f1,float f2) {
        
        int intRepresent1 = Float.floatToIntBits(f1);
        int intRepresent2 = Float.floatToIntBits(f2);
        
        sendBuffer[0]  = myID;
        
        sendBuffer[1]  =(byte) (intRepresent1      );
        sendBuffer[2]  =(byte) (intRepresent1 >> 8 );
        sendBuffer[3]  =(byte) (intRepresent1 >> 16);
        sendBuffer[4]  =(byte) (intRepresent1 >> 24);
        
        sendBuffer[5]  =(byte) (intRepresent2      );
        sendBuffer[6]  =(byte) (intRepresent2 >> 8 );
        sendBuffer[7]  =(byte) (intRepresent2 >> 16);
        sendBuffer[8]  =(byte) (intRepresent2 >> 24);
        
        newData=true;
    }
}